"""
Default ConfigUI for W8-E25D40 data recorders 
"""

from base64 import b64decode

DEFAULT_CONFIG_UI = b64decode(
    b'd3dZGkAoSDxQBYdHZW5lcmFsQAXzUAWLRGV2aWNlIE5hbWVQAYMI/39RJIFAUBXYQSBjdX'
    b'N0b20gbmFtZSBmb3IgdGhlIHJlY29yZGVyLiBOb3QgdGhlIHNhbWUgYXMgdGhlIHZvbHVt'
    b'ZSBsYWJlbC4gNjQgY2hhcmFjdGVycyBtYXhpbXVtLkAF81AFjERldmljZSBOb3Rlc1ABgw'
    b'n/f1EkggEAUBXSQ3VzdG9tIG5vdGVzIGFib3V0IHRoZSByZWNvcmRlciAocG9zaXRpb24s'
    b'IHVzZXIgSUQsIGV0Yy4pLiAyNTYgY2hhcmFjdGVycyBtYXhpbXVtLlE0gQNABUCZUAWVQ3'
    b'VzdG9tIFJlY29yZGluZyBUYWdzUAGDF/9/UQSAUSSBQFAV8UtleXdvcmRzIGZvciBvcmdh'
    b'bml6aW5nIGFuZCBpZGVudGlmeWluZyByZWNvcmRpbmdzLCBzZXBhcmF0ZWQgYnkgY29tbW'
    b'Egb3Igc3BhY2UuIExpbWl0IGlzIDIwIHRhZ3MsIDY0IGNoYXJhY3RlcnMuQAX7UAWTUmVj'
    b'b3JkaW5nIERpcmVjdG9yeVABgxT/f1EEhlJFQ09SRFEkgRBQFJNDb25maWdbMHgxNkZGN0'
    b'ZdPT0wUBW5RGlyZWN0b3J5IGZvciBzZXJpYWxpemVkIHJlY29yZGluZ3MuIDE2IGNoYXJh'
    b'Y3RlciBtYXhpbXVtQAVAo1AFlVJlY29yZGluZyBGaWxlIFByZWZpeFABgxX/f1EkgRBQFe'
    b'hQcmVmaXggZm9yIHNlcmlhbGl6ZWQgcmVjb3JkaW5nIG5hbWVzLiBJZiBibGFuaywgZGV2'
    b'aWNlIHNlcmlhbCBudW1iZXIgd2lsbCBiZSB1c2VkLiAxNiBjaGFyYWN0ZXIgbWF4aW11bV'
    b'AUk0NvbmZpZ1sweDE2RkY3Rl09PTBAB0E8UAWPRmlsZSBOYW1lIFN0eWxlUAGDFv9/UQGB'
    b'AVAVQPxUaW1lLUJhc2VkIGZpbGUgbmFtZXMgdXNlIGEgZGlyZWN0b3J5IGJhc2VkIG9uIH'
    b'RoZSBkYXRlLCBhbmQgZmlsZSBuYW1lIGJhc2VkIG9uIHRoZSBzZWNvbmRzIHNpbmNlIG1p'
    b'ZG5pZ2h0LgpTZXJpYWxpemVkIGZpbGUgbmFtZXMgdXNlIHRoZSBzcGVjaWZpZWQgZGlyZW'
    b'N0b3J5IGFuZCBmaWxlIG5hbWUgcHJlZml4LCBmb2xsb3dlZCBieSBhIG51bWJlci4gCk51'
    b'bWJlciBpcyByZXNldCB3aGVuIGRpcmVjdG9yeSBvciBwcmVmaXggY2hhbmdlcy5BB41QBY'
    b'pUaW1lIEJhc2VkQQeNUAWKU2VyaWFsaXplZEAHQKlQBYtCdXR0b24gTW9kZVABgxD/f1EB'
    b'gQBQFb9PbmUgU2Vjb25kIFByZXNzIHdpbGwgZGlzYWJsZSAzLXNlY29uZCBwcmVzcyBmb3'
    b'IgYmF0dGVyeSBjaGVjay5BB5dQBZRJbW1lZGlhdGUgU3RhcnQvU3RvcEEHolAFn09uZSBT'
    b'ZWNvbmQgUHJlc3MgZm9yIFN0YXJ0L1N0b3BBB41QBYpTdGFydC1Pbmx5QBdBFFAFjlBsdW'
    b'ctSW4gQWN0aW9uUAGDCv9/UQGBAVAVrVNldCBhY3Rpb24gZm9yIGRldmljZSB3aGVuIHBs'
    b'dWdnZWQgaW4gdG8gVVNCLkEHtVAFskltbWVkaWF0ZWx5IHN0b3AgcmVjb3JkaW5nIGFuZC'
    b'BhcHBlYXIgYXMgVVNCIGRyaXZlQQezUAWwQ29tcGxldGUgcmVjb3JkaW5nIGJlZm9yZSBh'
    b'cHBlYXJpbmcgYXMgVVNCIGRyaXZlQQewUAWtSWdub3JlOiBzdG9wIHJlY29yZGluZyB3aG'
    b'VuIGJ1dHRvbiBpcyBwcmVzc2VkQQelUAWiU3RhcnQgcmVjb3JkaW5nIHdoZW4gVVNCIGNv'
    b'bm5lY3RlZEAj51ABgwv/f1AV3kVudGVyIGxvY2FsIHRpbWV6b25lJ3Mgb2Zmc2V0IGZyb2'
    b'0gVVRDIHRpbWUuIEltcGFjdHMgdGltZXN0YW1wIGluZm9ybWF0aW9uIG9uIHJlY29yZGlu'
    b'ZyBmaWxlcy5AL4BAGEIIUAWMRW5hYmxlIFdpLUZpUAGDGP9/UQGBAUAn7FABgxn/f1EBgQ'
    b'FBB6lQBaZVcGxvYWQgcmVjb3JkaW5nIHRvIGNsb3VkIGFmdGVyIGZpbmlzaFAVs1NlbGVj'
    b'dCB0byB1cGxvYWQgZWFjaCBmaWxlIHdoZW4gcmVjb3JkaW5nIGNvbXBsZXRlc0AnQLZQAY'
    b'Ma/39RAYEAQQejUAWgRGVsZXRlIGZyb20gc3RvcmFnZSBhZnRlciB1cGxvYWRQFUCCUmVj'
    b'b3JkaW5nIGZpbGUgd2lsbCBiZSBkZWxldGVkIGFmdGVyIGEgc3VjY2Vzc2Z1bCB1cGxvYW'
    b'QsIGFuZCBtb3ZlZCB0byBhIHN1YmZvbGRlciBsYWJlbGVkICdmYWlsZWQnIGluIHRoZSBj'
    b'YXNlIG9mIGEgZmFpbGVkIHVwbG9hZEAnQMJQAYMd/39RAYEAQQepUAWmS2VlcCB0aGUgV2'
    b'ktRmkgbW9kdWxlIG9uIGFuZCBjb25uZWN0ZWRQFUCISWYgY2hlY2tlZCwgdGhlIFdpLUZp'
    b'IG1vZHVsZSB3aWxsIHN0YXkgb24gYW5kIGNvbm5lY3RlZC4gIEFsbG93cyBmb3IgZmFzdG'
    b'VyIHVwbG9hZCB0aW1lcyB3aXRoIHRoZSBleHBlbnNlIG9mIGluY3JlYXNlZCBwb3dlciBj'
    b'b25zdW1wdGlvbkAPgEBPgEAoSOdQBYhUcmlnZ2Vyc0AHQNpQBYxUcmlnZ2VyIE1vZGVQAY'
    b'MS/39RAYEAUBVAj0RlbGF5IFRoZW4gVHJpZ2dlcjogV2FpdCBzcGVjaWZpZWQgdGltZSwg'
    b'dGhlbiB3YWl0IGZvciB0cmlnZ2VyCkRlbGF5IE9yIFRyaWdnZXI6IEFjcXVpcmUgYWZ0ZX'
    b'Igd2FpdCBvciBvbiB0cmlnZ2VyLiBCYXR0ZXJ5IGxpZmUgbWF5IGJlIHJlZHVjZWQuQQeV'
    b'UAWSRGVsYXkgVGhlbiBUcmlnZ2VyQQeTUAWQRGVsYXkgT3IgVHJpZ2dlckAy9FAFjVN0YX'
    b'J0IGF0IFRpbWVQAYMP/39QFdtXYWl0IHVudGlsIHNldCB0aW1lIGJlZm9yZSBldmFsdWF0'
    b'aW5nIG90aGVyIHRyaWdnZXIgY29uZGl0aW9ucywgaW5jbHVkaW5nIGEgcmVjb3JkaW5nIG'
    b'RlbGF5QBGuUAWUUmVjb3JkaW5nIFRpbWUgTGltaXRQAYMN/39RIYT////wUCWHU2Vjb25k'
    b'c0ARQJBQBZhSZWNvcmRpbmcgRGVsYXkvSW50ZXJ2YWxQAYMM/39RAYEAUCWHU2Vjb25kc1'
    b'EhgwFRgFAV2FRpbWUgYmVmb3JlIHN0YXJ0aW5nIGEgcmVjb3JkaW5nLCBhbmQgdGltZSBi'
    b'ZXR3ZWVuIHJlY29yZGluZ3MgaWYgJ1JldHJpZ2dlcicgaXMgY2hlY2tlZC5AEa9QBZlSZW'
    b'NvcmRpbmcgRmlsZSBTaXplIExpbWl0UAGDEf9/USGE////8FAlg0tpQkAQQJdQBYlSZXRy'
    b'aWdnZXJQAYMO/39QFLBDb25maWdbMHhERkY3Rl09PW51bGwgYW5kIENvbmZpZ1sweDExRk'
    b'Y3Rl09PW51bGxQFc9XaGVuIHNldCwgU2xhbSBTdGljayB3aWxsIHJlLWFybSBhZnRlciBy'
    b'ZWNvcmRpbmcgdGltZSBvciBzaXplIGxpbWl0IGlzIHJlYWNoZWQuQBBAqFAFnldhaXQgZm'
    b'9yIEFsbCBTZW5zb3IgQ29uZGl0aW9uc1ABgxP/f1AV/ldoZW4gc2V0LCByZWNvcmRpbmcg'
    b'd2lsbCBvbmx5IHN0YXJ0IHdoZW4gYWxsIHRyaWdnZXJzIGFyZSBtZXQsIG90aGVyd2lzZS'
    b'ByZWNvcmRpbmcgd2lsbCBzdGFydCB3aGVuIGFueSBzZW5zb3IgY29uZGl0aW9uIGlzIG1l'
    b'dEAYQO5QBZBQcmVzc3VyZSBUcmlnZ2VyUAGDBQAkUQCBAECC5VAFlVByZXNzdXJlIFRyaW'
    b'dnZXIsIExvd1ABgwMAJFEBgwFfkFAlglBhUSGDAdTAUBWzU2V0IHRvIHN0YXJ0IHNhbXBs'
    b'aW5nIHdoZW4gb3V0c2lkZSBwcmVzc3VyZSB3aW5kb3cuQILmUAWWUHJlc3N1cmUgVHJpZ2'
    b'dlciwgSGlnaFABgwQAJFEBgwGtsFAlglBhUSGDAdTAUBWzU2V0IHRvIHN0YXJ0IHNhbXBs'
    b'aW5nIHdoZW4gb3V0c2lkZSBwcmVzc3VyZSB3aW5kb3cuQBhA+1AFk1RlbXBlcmF0dXJlIF'
    b'RyaWdnZXJQAYMFASRRAIEAQGPqUAWYVGVtcGVyYXR1cmUgVHJpZ2dlciwgTG93UAGDAwEk'
    b'UQKB9lAlgUNREoHYUSKBUFAVtlNldCB0byBzdGFydCBzYW1wbGluZyB3aGVuIG91dHNpZG'
    b'UgdGVtcGVyYXR1cmUgd2luZG93LkBj61AFmVRlbXBlcmF0dXJlIFRyaWdnZXIsIEhpZ2hQ'
    b'AYMEASRRAoEjUCWBQ1ESgdhRIoFQUBW2U2V0IHRvIHN0YXJ0IHNhbXBsaW5nIHdoZW4gb3'
    b'V0c2lkZSB0ZW1wZXJhdHVyZSB3aW5kb3cuQBhBp1AFmkFjY2VsZXJhdGlvbiBUcmlnZ2Vy'
    b'ICgyNWcpUAGDBf8IUBGDB/8gUQCBAFAkgFAUlUNvbmZpZ1sweDAxRkYwOF0gPT0gMEBDQK'
    b'tRE4jAOQAAAAAAAFEjiEA5AAAAAAAAUAWZQWNjZWxlcmF0aW9uIFRyaWdnZXIsIExvd1AB'
    b'gwP/CFECgvmaUBXgSGlnaCBwb3dlciB0cmlnZ2VyIHdpdGggcHJlLXRyaWdnZXIgZGF0YS'
    b'4gTm90IHJlY29tbWVuZGVkIGZvciBsb25nIHdhaXRzIHdpdGhvdXQgZXh0ZXJuYWwgcG93'
    b'ZXIuUgOIP0kAGQAZABlAQ0CsUROIwDkAAAAAAABRI4hAOQAAAAAAAFAFmkFjY2VsZXJhdG'
    b'lvbiBUcmlnZ2VyLCBIaWdoUAGDBP8IUQKCBmZQFeBIaWdoIHBvd2VyIHRyaWdnZXIgd2l0'
    b'aCBwcmUtdHJpZ2dlciBkYXRhLiBOb3QgcmVjb21tZW5kZWQgZm9yIGxvbmcgd2FpdHMgd2'
    b'l0aG91dCBleHRlcm5hbCBwb3dlci5SA4g/SQAZABkAGUAYQaNQBZpBY2NlbGVyYXRpb24g'
    b'VHJpZ2dlciAoNDBnKVABgwf/UFAV1ExvdyBwb3dlciB0cmlnZ2VyIHVzaW5nIGRpZ2l0YW'
    b'wgYWNjZWxlcm9tZXRlciB3aXRoIERDIHJlc3BvbnNlLiBObyBwcmUtdHJpZ2dlciBkYXRh'
    b'LlEAgQBQEYMF/whQEYMH/yBQJIBQFJRDb25maWdbMHgxZmY1MF0gPT0gMEBDQLJQBZZBY2'
    b'NlbGVyYXRpb24gVGhyZXNob2xkUAGDBP9QUQGCIABSA4g/RAAUABQAFFEjiEBEAAAAAAAA'
    b'UROAUBXyTG93IHBvd2VyIHRyaWdnZXIuIFRoZSBtaW5pbXVtIGFjY2VsZXJhdGlvbiAocG'
    b'9zaXRpdmUgb3IgbmVnYXRpdmUpIHRvIHRyaWdnZXIgYSByZWNvcmRpbmcuIE11c3QgYmUg'
    b'Z3JlYXRlciB0aGFuIDAuQCfGUAGDBf9QUQGBB0EHkVAFjlggQXhpcyBUcmlnZ2VyQQeRUA'
    b'WOWSBBeGlzIFRyaWdnZXJBB5FQBY5aIEF4aXMgVHJpZ2dlckAPgEBPgEAoR+JQBYxNZWFz'
    b'dXJlbWVudHNACEJaUAWXMjVnIEFjY2VsZXJhdGlvbiAoQ2ggOClQJIBAJ0CbUQGBD1ABgw'
    b'H/CFAVoEVuYWJsZS9EaXNhYmxlIGFjY2VsZXJhdGlvbiBheGVzQQeYUAWRRW5hYmxlIFgg'
    b'KENoIDguMClRAYEAQQeYUAWRRW5hYmxlIFkgKENoIDguMSlRAYEBQQeYUAWRRW5hYmxlIF'
    b'ogKENoIDguMilRAYECQQeaUAWTRW5hYmxlIE1pYyAoQ2ggOC4zKVEBgQNAEUCaUAWLU2Ft'
    b'cGxlIFJhdGVQAYMC/whQFdhTZXQgbWFpbiBhY2NlbGVyYXRpb24gc2FtcGxlIHJhdGUgZn'
    b'JvbSAxMCB0byAyMDAwMCBIei4KSGlnaGVyIHJhdGUgcmVkdWNlcyBiYXR0ZXJ5IGxpZmUu'
    b'UBSVQ29uZmlnWzB4MDFGRjA4XSA9PSAwUQGCE4hQJYJIelERgQpRIYJOIEARQPxQBaNPdm'
    b'VycmlkZSBBbnRpYWxpYXNpbmcgRmlsdGVyIEN1dG9mZlABgwj/CFEBggPoUCWCSHpRIYJh'
    b'qFAVQKVEZWZhdWx0LCB3aGVuIG5vdCBjaGVja2VkLCBvZiBvbmUtZmlmdGggc2FtcGxlIG'
    b'ZyZXF1ZW5jeS4gQ3V0LW9mZiBmcmVxdWVuY3kgaGFzIDI5JSBhdHRlbnVhdGlvbi4KNXRo'
    b'IG9yZGVyIEJ1dHRlcndvcnRoLiBTb21lIGF0dGVudWF0aW9uIGF0IDYwJSBvZiBjdXQtb2'
    b'ZmIGZyZXF1ZW5jeS5QFJVDb25maWdbMHgwMUZGMDhdID09IDBACEEcUAWbNDBnIERDIEFj'
    b'Y2VsZXJhdGlvbiAoQ2ggODApQCefUAGDAf9QUQGBAUEHklAFj0VuYWJsZSBBbGwgQXhlc0'
    b'AXQNhQBYtTYW1wbGUgUmF0ZVABgwL/UFEBggH0UCWCSHpQFd5Mb3cgcGFzcyBmaWx0ZXIg'
    b'aXMgc2V0IHRvIDI1JSBvZiBkYXRhIHJhdGUsIHNvIGEgZGF0YSByYXRlIG9mIDIwMDBIei'
    b'BoYXMgYSBMUCBmaWx0ZXIgYXQgNTAwIEh6UBSTbm90IENvbmZpZ1sweDFmZjUwXUEHhVEB'
    b'gg+gQQeFUQGCB9BBB4VRAYID6EEHhVEBggH0QQeEUQGB+kEHhFEBgX1BB4RRAYE/QQeEUQ'
    b'GBIEEHhFEBgRBACEEwUAWoQ29udHJvbCBQYWQgVGVtcGVyYXR1cmUvUHJlc3N1cmUgKENo'
    b'IDU5KUAnQJtRAYEDUAGDAf87UBWlRW5hYmxlL0Rpc2FibGUgc2Vuc29ycyBvbiBjb250cm'
    b'9sIHBhZEEHnFAFmUVuYWJsZSBQcmVzc3VyZSAoQ2ggNTkuMClBB59QBZxFbmFibGUgVGVt'
    b'cGVyYXR1cmUgKENoIDU5LjEpQQelUAWiRW5hYmxlIFJlbGF0aXZlIEh1bWlkaXR5IChDaC'
    b'A1OS4yKUAR41AFi1NhbXBsZSBSYXRlUAGDAv87UBSUQ29uZmlnWzB4MWZmM0JdID09IDBQ'
    b'FaRTZXQgc2FtcGxlIGZyZXF1ZW5jeSBmcm9tIDEgdG8gMTAgSHpRAYEKUCWCSHpREYEBUS'
    b'GBCkAIQc1QBZlJbmVydGlhbCBNZWFzdXJlbWVudCBVbml0QAdAllAFkEFjcXVpc2l0aW9u'
    b'IE1vZGVQAYMB/ytQJYEgUQGBEEEHilAFg09mZlEBgQBBB6hQBaFBYnNvbHV0ZSBPcmllbn'
    b'RhdGlvbiAoUXVhdGVybmlvbilRAYEIQQeoUAWhUmVsYXRpdmUgT3JpZW50YXRpb24gKFF1'
    b'YXRlcm5pb24pUQGBEEEHj1AFiFJvdGF0aW9uUQGBAkABQIZQBZVPcmllbnRhdGlvbiBEYX'
    b'RhIFJhdGVQAYMCCCtRAYFkUCWCSHpREYEBUSGByFAVvVNhbXBsZSByYXRlIGZvciBkaXJl'
    b'Y3Rpb25hbCBkYXRhIGZyb20gMSB0byAyMDAgSHouIENoYW5uZWwgNjVQFJRDb25maWdbMH'
    b'gwMUZGMkJdIDwgOEABQIlQBZJSb3RhdGlvbiBEYXRhIFJhdGVQAYMCAitRAYFkUCWCSHpR'
    b'EYEBUSGByFAVvlNldCBzYW1wbGUgcmF0ZSBmb3Igcm90YXRpb24gZGF0YSBmcm9tIDEgdG'
    b'8gMjAwIEh6LiBDaGFubmVsIDQ3UBSZQ29uZmlnWzB4MDFGRjJCXSAmIDIgPT0gMEAIQJBQ'
    b'BaBDb250cm9sIFBhZCBMaWdodCBTZW5zb3IgKENoIDc2KUAn6lABgwH/TFAVvkVuYWJsZS'
    b'9EaXNhYmxlIGxpZ2h0IHNlbnNvciBhdCA0IEh6IC0gTk9URTogRElTQUJMRUQgRk9SIEFM'
    b'UEhBQQegUAWdRW5hYmxlIExpZ2h0IFNlbnNvciAoQ2ggNzYuMClACECyUAWTQk1HMjUwIE'
    b'd5cm8gKENoIDg0KUAnn1ABgwH/VFEBgQBBB5JQBY9FbmFibGUgQWxsIEF4ZXNAF/dQBYtT'
    b'YW1wbGUgUmF0ZVABgwL/VFEBggGQUCWCSHpQFJNub3QgQ29uZmlnWzB4MWZmNTRdQQeFUQ'
    b'GCDIBBB4VRAYIGQEEHhVEBggMgQQeFUQGCAZBBB4RRAYHIQQeEUQGBZEEHhFEBgTJBB4RR'
    b'AYEZQQeEUQGBDEAPgEBPgEoIgEoogEpIgA=='
)



"""
Exceptions raised when interacting with a recording device.
"""

__all__ = ('CommandError', 'CommunicationError', 'ConfigError',
           'ConfigVersionError', 'DeviceError', 'DeviceTimeout',
           'UnsupportedFeature')


class DeviceError(Exception):
    """ Base class for device-related exceptions. """
    @property
    def errno(self):
        if len(self.args) > 1:
            return self.args[0]
        return None


class CommandError(RuntimeError, DeviceError):
    """ Exception raised by a failure to process a command. """


class CommunicationError(RuntimeError, DeviceError):
    """ Exception raised by a failure to communicate. """


class ConfigError(ValueError, DeviceError):
    """ Exception raised when configuration data is invalid.
    """


class ConfigVersionError(ConfigError, DeviceError):
    """ Exception raised when configuration format doesn't match the recorder
        hardware or firmware version.
    """


class DeviceTimeout(TimeoutError, DeviceError):
    """ Exception raised when a device fails to respond within an expected
        length of time.
    """


class UnsupportedFeature(DeviceError):
    """ Exception raised when a device does not support a given feature
        (e.g., attempting to execute Wi-Fi commands on a device without
        Wi-Fi, or executing a command exclusive to the serial command
        interface over the file-based interface).

        Intended to be instantiated with either a single argument (a message
        string) or with two (the object raising the exception, and the
        offending method).
    """
    @property
    def errno(self):
        return None

    def __str__(self):
        try:
            if len(self.args) == 2:
                return "{}.{}".format(type(self.args[0]).__name__,
                                      self.args[1].__name__)
        except (AttributeError, IndexError, TypeError):
            pass

        return super().__str__()


class CRCError(ValueError):
    """ Exception raised if a packet's CRC16 check fails. """



"""
MacOS-specific functions; primarily filesystem-related. The parent package
imports the appropriate version for the host OS.

TODO: THIS NEEDS TO BE REFACTORED. BASE ON NEXT RELEASE VERSION OF `linux.py`
"""

__all__ = ('deviceChanged', 'getDeviceList', 'getBlockSize', 'getFreeSpace',
           'getDriveInfo', 'readRecorderClock', 'readUncachedFile')

import os

from .linux import getBlockSize, getDriveInfo, readUncachedFile, readRecorderClock

#===============================================================================
#
#===============================================================================


def getDeviceList(types, paths=None):
    """ Get a list of data recorders, as their respective drive letter.
    """
    paths = os.listdir("/Volumes/") if paths is None else paths
    paths = filter(lambda x: x not in ("Mobile Backups", "Macintosh HD"), paths)
    result = []
    for p in paths:
        for t in types:
            if t.isRecorder(p):
                result.append(p)
    return result


_LAST_DEVICES = 0
_LAST_RECORDERS = None


def deviceChanged(recordersOnly, types, clear=False):
    """ Returns `True` if a drive has been connected or disconnected since
        the last call to `deviceChanged()`.

        :param recordersOnly: If `False`, any change to the mounted drives
            is reported as a change. If `True`, the mounted drives are checked
            and `True` is only returned if the change occurred to a recorder.
            Checking for recorders only takes marginally more time.
        :param types: A list of known `Recorder` classes to detect.
        :param clear: If `True`, clear the cache of previously-detected
            drives and devices.
    """
    global _LAST_DEVICES, _LAST_RECORDERS
    
    if clear:
        _LAST_DEVICES = 0
        _LAST_RECORDERS = None
    
    newDevices = os.listdir("/Volumes/")
    changed = newDevices != _LAST_DEVICES
    _LAST_DEVICES = newDevices

#     if not changed or not recordersOnly:
    if not recordersOnly:
        return changed

    newRecorders = tuple(getDeviceList(types=types))
    changed = newRecorders != _LAST_RECORDERS
    _LAST_RECORDERS = newRecorders
    return changed


def getFreeSpace(path):
    """ Return the free space (in bytes) on a drive.

        :param path: The path to the drive to check. Can be a subdirectory.
        :return: The free space on the drive, in bytes.
        :rtype: int
    """
    # TODO: Make sure this actually works. Should work on all POSIX OSes.
    st = os.statvfs(path)
    return st.f_bavail * st.f_frsize



"""
Default ConfigUI for S5-D40 data recorders 
"""

from base64 import b64decode

DEFAULT_CONFIG_UI = b64decode(
    b'd3dTfEAoRjBQBYdHZW5lcmFsQAXzUAWLRGV2aWNlIE5hbWVQAYMI/39RJIFAUBXYQSBjdX'
    b'N0b20gbmFtZSBmb3IgdGhlIHJlY29yZGVyLiBOb3QgdGhlIHNhbWUgYXMgdGhlIHZvbHVt'
    b'ZSBsYWJlbC4gNjQgY2hhcmFjdGVycyBtYXhpbXVtLkAF81AFjERldmljZSBOb3Rlc1ABgw'
    b'n/f1EkggEAUBXSQ3VzdG9tIG5vdGVzIGFib3V0IHRoZSByZWNvcmRlciAocG9zaXRpb24s'
    b'IHVzZXIgSUQsIGV0Yy4pLiAyNTYgY2hhcmFjdGVycyBtYXhpbXVtLlE0gQNABUCZUAWVQ3'
    b'VzdG9tIFJlY29yZGluZyBUYWdzUAGDF/9/UQSAUSSBQFAV8UtleXdvcmRzIGZvciBvcmdh'
    b'bml6aW5nIGFuZCBpZGVudGlmeWluZyByZWNvcmRpbmdzLCBzZXBhcmF0ZWQgYnkgY29tbW'
    b'Egb3Igc3BhY2UuIExpbWl0IGlzIDIwIHRhZ3MsIDY0IGNoYXJhY3RlcnMuQAX7UAWTUmVj'
    b'b3JkaW5nIERpcmVjdG9yeVABgxT/f1EEhlJFQ09SRFEkgRBQFJNDb25maWdbMHgxNkZGN0'
    b'ZdPT0wUBW5RGlyZWN0b3J5IGZvciBzZXJpYWxpemVkIHJlY29yZGluZ3MuIDE2IGNoYXJh'
    b'Y3RlciBtYXhpbXVtQAVAo1AFlVJlY29yZGluZyBGaWxlIFByZWZpeFABgxX/f1EkgRBQFe'
    b'hQcmVmaXggZm9yIHNlcmlhbGl6ZWQgcmVjb3JkaW5nIG5hbWVzLiBJZiBibGFuaywgZGV2'
    b'aWNlIHNlcmlhbCBudW1iZXIgd2lsbCBiZSB1c2VkLiAxNiBjaGFyYWN0ZXIgbWF4aW11bV'
    b'AUk0NvbmZpZ1sweDE2RkY3Rl09PTBAB0E8UAWPRmlsZSBOYW1lIFN0eWxlUAGDFv9/UQGB'
    b'AVAVQPxUaW1lLUJhc2VkIGZpbGUgbmFtZXMgdXNlIGEgZGlyZWN0b3J5IGJhc2VkIG9uIH'
    b'RoZSBkYXRlLCBhbmQgZmlsZSBuYW1lIGJhc2VkIG9uIHRoZSBzZWNvbmRzIHNpbmNlIG1p'
    b'ZG5pZ2h0LgpTZXJpYWxpemVkIGZpbGUgbmFtZXMgdXNlIHRoZSBzcGVjaWZpZWQgZGlyZW'
    b'N0b3J5IGFuZCBmaWxlIG5hbWUgcHJlZml4LCBmb2xsb3dlZCBieSBhIG51bWJlci4gCk51'
    b'bWJlciBpcyByZXNldCB3aGVuIGRpcmVjdG9yeSBvciBwcmVmaXggY2hhbmdlcy5BB41QBY'
    b'pUaW1lIEJhc2VkQQeNUAWKU2VyaWFsaXplZEAHQKlQBYtCdXR0b24gTW9kZVABgxD/f1EB'
    b'gQBQFb9PbmUgU2Vjb25kIFByZXNzIHdpbGwgZGlzYWJsZSAzLXNlY29uZCBwcmVzcyBmb3'
    b'IgYmF0dGVyeSBjaGVjay5BB5dQBZRJbW1lZGlhdGUgU3RhcnQvU3RvcEEHolAFn09uZSBT'
    b'ZWNvbmQgUHJlc3MgZm9yIFN0YXJ0L1N0b3BBB41QBYpTdGFydC1Pbmx5QBdBFFAFjlBsdW'
    b'ctSW4gQWN0aW9uUAGDCv9/UQGBAVAVrVNldCBhY3Rpb24gZm9yIGRldmljZSB3aGVuIHBs'
    b'dWdnZWQgaW4gdG8gVVNCLkEHtVAFskltbWVkaWF0ZWx5IHN0b3AgcmVjb3JkaW5nIGFuZC'
    b'BhcHBlYXIgYXMgVVNCIGRyaXZlQQezUAWwQ29tcGxldGUgcmVjb3JkaW5nIGJlZm9yZSBh'
    b'cHBlYXJpbmcgYXMgVVNCIGRyaXZlQQewUAWtSWdub3JlOiBzdG9wIHJlY29yZGluZyB3aG'
    b'VuIGJ1dHRvbiBpcyBwcmVzc2VkQQelUAWiU3RhcnQgcmVjb3JkaW5nIHdoZW4gVVNCIGNv'
    b'bm5lY3RlZEAj51ABgwv/f1AV3kVudGVyIGxvY2FsIHRpbWV6b25lJ3Mgb2Zmc2V0IGZyb2'
    b'0gVVRDIHRpbWUuIEltcGFjdHMgdGltZXN0YW1wIGluZm9ybWF0aW9uIG9uIHJlY29yZGlu'
    b'ZyBmaWxlcy5AL4BAD4BAT4BAKEd0UAWIVHJpZ2dlcnNAB0DaUAWMVHJpZ2dlciBNb2RlUA'
    b'GDEv9/UQGBAFAVQI9EZWxheSBUaGVuIFRyaWdnZXI6IFdhaXQgc3BlY2lmaWVkIHRpbWUs'
    b'IHRoZW4gd2FpdCBmb3IgdHJpZ2dlcgpEZWxheSBPciBUcmlnZ2VyOiBBY3F1aXJlIGFmdG'
    b'VyIHdhaXQgb3Igb24gdHJpZ2dlci4gQmF0dGVyeSBsaWZlIG1heSBiZSByZWR1Y2VkLkEH'
    b'lVAFkkRlbGF5IFRoZW4gVHJpZ2dlckEHk1AFkERlbGF5IE9yIFRyaWdnZXJAMvRQBY1TdG'
    b'FydCBhdCBUaW1lUAGDD/9/UBXbV2FpdCB1bnRpbCBzZXQgdGltZSBiZWZvcmUgZXZhbHVh'
    b'dGluZyBvdGhlciB0cmlnZ2VyIGNvbmRpdGlvbnMsIGluY2x1ZGluZyBhIHJlY29yZGluZy'
    b'BkZWxheUARrlAFlFJlY29yZGluZyBUaW1lIExpbWl0UAGDDf9/USGE////8FAlh1NlY29u'
    b'ZHNAEUCQUAWYUmVjb3JkaW5nIERlbGF5L0ludGVydmFsUAGDDP9/UQGBAFAlh1NlY29uZH'
    b'NRIYMBUYBQFdhUaW1lIGJlZm9yZSBzdGFydGluZyBhIHJlY29yZGluZywgYW5kIHRpbWUg'
    b'YmV0d2VlbiByZWNvcmRpbmdzIGlmICdSZXRyaWdnZXInIGlzIGNoZWNrZWQuQBGvUAWZUm'
    b'Vjb3JkaW5nIEZpbGUgU2l6ZSBMaW1pdFABgxH/f1EhhP////BQJYNLaUJAEECXUAWJUmV0'
    b'cmlnZ2VyUAGDDv9/UBSwQ29uZmlnWzB4REZGN0ZdPT1udWxsIGFuZCBDb25maWdbMHgxMU'
    b'ZGN0ZdPT1udWxsUBXPV2hlbiBzZXQsIFNsYW0gU3RpY2sgd2lsbCByZS1hcm0gYWZ0ZXIg'
    b'cmVjb3JkaW5nIHRpbWUgb3Igc2l6ZSBsaW1pdCBpcyByZWFjaGVkLkAQQKhQBZ5XYWl0IG'
    b'ZvciBBbGwgU2Vuc29yIENvbmRpdGlvbnNQAYMT/39QFf5XaGVuIHNldCwgcmVjb3JkaW5n'
    b'IHdpbGwgb25seSBzdGFydCB3aGVuIGFsbCB0cmlnZ2VycyBhcmUgbWV0LCBvdGhlcndpc2'
    b'UgcmVjb3JkaW5nIHdpbGwgc3RhcnQgd2hlbiBhbnkgc2Vuc29yIGNvbmRpdGlvbiBpcyBt'
    b'ZXRAGEEKUAWQUHJlc3N1cmUgVHJpZ2dlclABgwUAJFEAgQBQFJlDb25maWdbMHgwMUZGMj'
    b'RdICYgMSA9PSAwQILlUAWVUHJlc3N1cmUgVHJpZ2dlciwgTG93UAGDAwAkUQGDAV+QUCWC'
    b'UGFRIYMB1MBQFbNTZXQgdG8gc3RhcnQgc2FtcGxpbmcgd2hlbiBvdXRzaWRlIHByZXNzdX'
    b'JlIHdpbmRvdy5AguZQBZZQcmVzc3VyZSBUcmlnZ2VyLCBIaWdoUAGDBAAkUQGDAa2wUCWC'
    b'UGFRIYMB1MBQFbNTZXQgdG8gc3RhcnQgc2FtcGxpbmcgd2hlbiBvdXRzaWRlIHByZXNzdX'
    b'JlIHdpbmRvdy5AGEEXUAWTVGVtcGVyYXR1cmUgVHJpZ2dlclABgwUBJFEAgQBQFJlDb25m'
    b'aWdbMHgwMUZGMjRdICYgMiA9PSAwQGPqUAWYVGVtcGVyYXR1cmUgVHJpZ2dlciwgTG93UA'
    b'GDAwEkUQKB9lAlgUNREoHYUSKBUFAVtlNldCB0byBzdGFydCBzYW1wbGluZyB3aGVuIG91'
    b'dHNpZGUgdGVtcGVyYXR1cmUgd2luZG93LkBj61AFmVRlbXBlcmF0dXJlIFRyaWdnZXIsIE'
    b'hpZ2hQAYMEASRRAoEjUCWBQ1ESgdhRIoFQUBW2U2V0IHRvIHN0YXJ0IHNhbXBsaW5nIHdo'
    b'ZW4gb3V0c2lkZSB0ZW1wZXJhdHVyZSB3aW5kb3cuQBhBo1AFmkFjY2VsZXJhdGlvbiBUcm'
    b'lnZ2VyICg0MGcpUAGDB/9QUBXUTG93IHBvd2VyIHRyaWdnZXIgdXNpbmcgZGlnaXRhbCBh'
    b'Y2NlbGVyb21ldGVyIHdpdGggREMgcmVzcG9uc2UuIE5vIHByZS10cmlnZ2VyIGRhdGEuUQ'
    b'CBAFARgwX/CFARgwf/IFAkgFAUlENvbmZpZ1sweDFmZjUwXSA9PSAwQENAslAFlkFjY2Vs'
    b'ZXJhdGlvbiBUaHJlc2hvbGRQAYME/1BRAYIgAFIDiD9EABQAFAAUUSOIQEQAAAAAAABRE4'
    b'BQFfJMb3cgcG93ZXIgdHJpZ2dlci4gVGhlIG1pbmltdW0gYWNjZWxlcmF0aW9uIChwb3Np'
    b'dGl2ZSBvciBuZWdhdGl2ZSkgdG8gdHJpZ2dlciBhIHJlY29yZGluZy4gTXVzdCBiZSBncm'
    b'VhdGVyIHRoYW4gMC5AJ8ZQAYMF/1BRAYEHQQeRUAWOWCBBeGlzIFRyaWdnZXJBB5FQBY5Z'
    b'IEF4aXMgVHJpZ2dlckEHkVAFjlogQXhpcyBUcmlnZ2VyQA+AQE+AQChFw1AFjE1lYXN1cm'
    b'VtZW50c0AIQRxQBZs0MGcgREMgQWNjZWxlcmF0aW9uIChDaCA4MClAJ59QAYMB/1BRAYEB'
    b'QQeSUAWPRW5hYmxlIEFsbCBBeGVzQBdA2FAFi1NhbXBsZSBSYXRlUAGDAv9QUQGCAfRQJY'
    b'JIelAV3kxvdyBwYXNzIGZpbHRlciBpcyBzZXQgdG8gMjUlIG9mIGRhdGEgcmF0ZSwgc28g'
    b'YSBkYXRhIHJhdGUgb2YgMjAwMEh6IGhhcyBhIExQIGZpbHRlciBhdCA1MDAgSHpQFJNub3'
    b'QgQ29uZmlnWzB4MWZmNTBdQQeFUQGCD6BBB4VRAYIH0EEHhVEBggPoQQeFUQGCAfRBB4RR'
    b'AYH6QQeEUQGBfUEHhFEBgT9BB4RRAYEgQQeEUQGBEEAIQTBQBahDb250cm9sIFBhZCBUZW'
    b'1wZXJhdHVyZS9QcmVzc3VyZSAoQ2ggNTkpQCdAm1EBgQNQAYMB/ztQFaVFbmFibGUvRGlz'
    b'YWJsZSBzZW5zb3JzIG9uIGNvbnRyb2wgcGFkQQecUAWZRW5hYmxlIFByZXNzdXJlIChDaC'
    b'A1OS4wKUEHn1AFnEVuYWJsZSBUZW1wZXJhdHVyZSAoQ2ggNTkuMSlBB6VQBaJFbmFibGUg'
    b'UmVsYXRpdmUgSHVtaWRpdHkgKENoIDU5LjIpQBHjUAWLU2FtcGxlIFJhdGVQAYMC/ztQFJ'
    b'RDb25maWdbMHgxZmYzQl0gPT0gMFAVpFNldCBzYW1wbGUgZnJlcXVlbmN5IGZyb20gMSB0'
    b'byAxMCBIelEBgQpQJYJIelERgQFRIYEKQAhBzVAFmUluZXJ0aWFsIE1lYXN1cmVtZW50IF'
    b'VuaXRAB0CWUAWQQWNxdWlzaXRpb24gTW9kZVABgwH/K1AlgSBRAYEQQQeKUAWDT2ZmUQGB'
    b'AEEHqFAFoUFic29sdXRlIE9yaWVudGF0aW9uIChRdWF0ZXJuaW9uKVEBgQhBB6hQBaFSZW'
    b'xhdGl2ZSBPcmllbnRhdGlvbiAoUXVhdGVybmlvbilRAYEQQQePUAWIUm90YXRpb25RAYEC'
    b'QAFAhlAFlU9yaWVudGF0aW9uIERhdGEgUmF0ZVABgwIIK1EBgWRQJYJIelERgQFRIYHIUB'
    b'W9U2FtcGxlIHJhdGUgZm9yIGRpcmVjdGlvbmFsIGRhdGEgZnJvbSAxIHRvIDIwMCBIei4g'
    b'Q2hhbm5lbCA2NVAUlENvbmZpZ1sweDAxRkYyQl0gPCA4QAFAiVAFklJvdGF0aW9uIERhdG'
    b'EgUmF0ZVABgwICK1EBgWRQJYJIelERgQFRIYHIUBW+U2V0IHNhbXBsZSByYXRlIGZvciBy'
    b'b3RhdGlvbiBkYXRhIGZyb20gMSB0byAyMDAgSHouIENoYW5uZWwgNDdQFJlDb25maWdbMH'
    b'gwMUZGMkJdICYgMiA9PSAwQAhA8VAFpUludGVybmFsIFRlbXBlcmF0dXJlL1ByZXNzdXJl'
    b'IChDaCAzNilAJ0DFUQGBA1ABgwH/JFAV90VuYWJsZS9EaXNhYmxlIHNsb3dlciBpbnRlcm'
    b'5hbCBlbnZpcm9ubWVudGFsIHNlbnNvcnMuIE5PVEU6IFRlbXBlcmF0dXJlIGlzIHJlcXVp'
    b'cmVkIGZvciBtYWluIGFjY2VsZXJvbWV0ZXIgbWVhc3VyZW1lbnRzQQecUAWZRW5hYmxlIF'
    b'ByZXNzdXJlIChDaCAzNi4wKUEHn1AFnEVuYWJsZSBUZW1wZXJhdHVyZSAoQ2ggMzYuMSlA'
    b'CECQUAWgQ29udHJvbCBQYWQgTGlnaHQgU2Vuc29yIChDaCA3NilAJ+pQAYMB/0xQFb5Fbm'
    b'FibGUvRGlzYWJsZSBsaWdodCBzZW5zb3IgYXQgNCBIeiAtIE5PVEU6IERJU0FCTEVEIEZP'
    b'UiBBTFBIQUEHoFAFnUVuYWJsZSBMaWdodCBTZW5zb3IgKENoIDc2LjApQA+AQE+ASgiASi'
    b'iASkiA'
)



import codecs
import os.path
import setuptools


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


INSTALL_REQUIRES = [
    'ebmlite>=3.3.0',
    'idelib>=3.2.9',
    'numpy>=1.19.4',
    'psutil >=5.5.0, <6.0.0; sys_platform == "linux" or sys_platform=="darwin"',
    'pyserial>=3.5',
    'pywin32>=228; sys_platform == "win32"'
]

TEST_REQUIRES = [
    'pytest>=7.2',
]

DOCS_REQUIRES = [
    'Sphinx==4.5.0',
    'sphinxcontrib-applehelp==1.0.2',
    'sphinxcontrib-devhelp==1.0.2',
    'sphinxcontrib-htmlhelp==2.0.0',
    'sphinxcontrib-jsmath==1.0.1',
    'sphinxcontrib-qthelp==1.0.3',
    'sphinxcontrib-serializinghtml==1.1.5',
    'pydata-sphinx-theme==0.7.1',
    'sphinx-autodoc-typehints==1.18.1',
]


setuptools.setup(
        name='endaq-device',
        version=get_version('endaq/device/__init__.py'),
        author='Mide Technology',
        author_email='help@mide.com',
        description='Python API for enDAQ data recorders',
        long_description=read('README.md'),
        long_description_content_type='text/markdown',
        url='https://github.com/MideTechnology/endaq-device',
        license='MIT',
        classifiers=['Development Status :: 5 - Production/Stable',
                     'License :: OSI Approved :: MIT License',
                     'Natural Language :: English',
                     'Programming Language :: Python :: 3.8',
                     'Programming Language :: Python :: 3.9',
                     'Programming Language :: Python :: 3.10',
                     'Programming Language :: Python :: 3.11',
                     'Programming Language :: Python :: 3.12',
                     ],
        keywords='endaq configure recorder hardware',
        project_urls={
            "Bug Tracker": "https://github.com/MideTechnology/endaq-device/issues",
            "Documentation": "https://mide-technology-endaq-device.readthedocs-hosted.com/en/latest/",
            "Source Code": "https://github.com/MideTechnology/endaq-device",
            },
        packages=[
            'endaq.device',
            'endaq.device.ui_defaults',
            'endaq.device.schemata',
            ],
        package_dir={
            'endaq.device': './endaq/device',
            },
        package_data={
            '': ['schemata/*.xml'],
        },
        test_suite='tests',
        install_requires=[
            'idelib>=3.2',
            'numpy>=1.19.4',
            'ebmlite>=3.1.0',
            'psutil>=5.5.0, <6.0.0; sys_platform == "linux" or sys_platform=="darwin"',
            'pyserial>=3.5',
            'pywin32>=228; sys_platform == "win32"'
            ],
        extras_require={
            'test': INSTALL_REQUIRES + TEST_REQUIRES,
            'docs': INSTALL_REQUIRES + DOCS_REQUIRES,
            },
        # tests_require=[
        #     'pytest',
        #     'mock'
        #     ],
)



"""
Default ConfigUI for LOG-0004-500G-DC data recorders 
"""

from base64 import b64decode

DEFAULT_CONFIG_UI = b64decode(
    b'd3dVDEAoRYFQBYdHZW5lcmFsQAXzUAWLRGV2aWNlIE5hbWVQAYMI/39RJIFAUBXYQSBjdX'
    b'N0b20gbmFtZSBmb3IgdGhlIHJlY29yZGVyLiBOb3QgdGhlIHNhbWUgYXMgdGhlIHZvbHVt'
    b'ZSBsYWJlbC4gNjQgY2hhcmFjdGVycyBtYXhpbXVtLkAF81AFjERldmljZSBOb3Rlc1ABgw'
    b'n/f1EkggEAUBXSQ3VzdG9tIG5vdGVzIGFib3V0IHRoZSByZWNvcmRlciAocG9zaXRpb24s'
    b'IHVzZXIgSUQsIGV0Yy4pLiAyNTYgY2hhcmFjdGVycyBtYXhpbXVtLlE0gQNABfpQBZNSZW'
    b'NvcmRpbmcgRGlyZWN0b3J5UAGDFP9/UQSGUkVDT1JEUSSBCFAUk0NvbmZpZ1sweDE2RkY3'
    b'Rl09PTBQFbhEaXJlY3RvcnkgZm9yIHNlcmlhbGl6ZWQgcmVjb3JkaW5ncy4gOCBjaGFyYW'
    b'N0ZXIgbWF4aW11bUAFQKJQBZVSZWNvcmRpbmcgRmlsZSBQcmVmaXhQAYMV/39RJIEIUBXn'
    b'UHJlZml4IGZvciBzZXJpYWxpemVkIHJlY29yZGluZyBuYW1lcy4gSWYgYmxhbmssIGRldm'
    b'ljZSBzZXJpYWwgbnVtYmVyIHdpbGwgYmUgdXNlZC4gOCBjaGFyYWN0ZXIgbWF4aW11bVAU'
    b'k0NvbmZpZ1sweDE2RkY3Rl09PTBAB0E8UAWPRmlsZSBOYW1lIFN0eWxlUAGDFv9/UQGBAF'
    b'AVQPxUaW1lLUJhc2VkIGZpbGUgbmFtZXMgdXNlIGEgZGlyZWN0b3J5IGJhc2VkIG9uIHRo'
    b'ZSBkYXRlLCBhbmQgZmlsZSBuYW1lIGJhc2VkIG9uIHRoZSBzZWNvbmRzIHNpbmNlIG1pZG'
    b'5pZ2h0LgpTZXJpYWxpemVkIGZpbGUgbmFtZXMgdXNlIHRoZSBzcGVjaWZpZWQgZGlyZWN0'
    b'b3J5IGFuZCBmaWxlIG5hbWUgcHJlZml4LCBmb2xsb3dlZCBieSBhIG51bWJlci4gCk51bW'
    b'JlciBpcyByZXNldCB3aGVuIGRpcmVjdG9yeSBvciBwcmVmaXggY2hhbmdlcy5BB41QBYpU'
    b'aW1lIEJhc2VkQQeNUAWKU2VyaWFsaXplZEAHQJlQBYtCdXR0b24gTW9kZVABgxD/f1EBgQ'
    b'BQFb9PbmUgU2Vjb25kIFByZXNzIHdpbGwgZGlzYWJsZSAzLXNlY29uZCBwcmVzcyBmb3Ig'
    b'YmF0dGVyeSBjaGVjay5BB5dQBZRJbW1lZGlhdGUgU3RhcnQvU3RvcEEHolAFn09uZSBTZW'
    b'NvbmQgUHJlc3MgZm9yIFN0YXJ0L1N0b3BAF0EUUAWOUGx1Zy1JbiBBY3Rpb25QAYMK/39R'
    b'AYEBUBWtU2V0IGFjdGlvbiBmb3IgZGV2aWNlIHdoZW4gcGx1Z2dlZCBpbiB0byBVU0IuQQ'
    b'e1UAWySW1tZWRpYXRlbHkgc3RvcCByZWNvcmRpbmcgYW5kIGFwcGVhciBhcyBVU0IgZHJp'
    b'dmVBB7NQBbBDb21wbGV0ZSByZWNvcmRpbmcgYmVmb3JlIGFwcGVhcmluZyBhcyBVU0IgZH'
    b'JpdmVBB7BQBa1JZ25vcmU6IHN0b3AgcmVjb3JkaW5nIHdoZW4gYnV0dG9uIGlzIHByZXNz'
    b'ZWRBB6VQBaJTdGFydCByZWNvcmRpbmcgd2hlbiBVU0IgY29ubmVjdGVkQCPnUAGDC/9/UB'
    b'XeRW50ZXIgbG9jYWwgdGltZXpvbmUncyBvZmZzZXQgZnJvbSBVVEMgdGltZS4gSW1wYWN0'
    b'cyB0aW1lc3RhbXAgaW5mb3JtYXRpb24gb24gcmVjb3JkaW5nIGZpbGVzLkAvgEAPgEBPgE'
    b'AoSOFQBYhUcmlnZ2Vyc0AHQNpQBYxUcmlnZ2VyIE1vZGVQAYMS/39RAYEAUBVAj0RlbGF5'
    b'IFRoZW4gVHJpZ2dlcjogV2FpdCBzcGVjaWZpZWQgdGltZSwgdGhlbiB3YWl0IGZvciB0cm'
    b'lnZ2VyCkRlbGF5IE9yIFRyaWdnZXI6IEFjcXVpcmUgYWZ0ZXIgd2FpdCBvciBvbiB0cmln'
    b'Z2VyLiBCYXR0ZXJ5IGxpZmUgbWF5IGJlIHJlZHVjZWQuQQeVUAWSRGVsYXkgVGhlbiBUcm'
    b'lnZ2VyQQeTUAWQRGVsYXkgT3IgVHJpZ2dlckAy9FAFjVN0YXJ0IGF0IFRpbWVQAYMP/39Q'
    b'FdtXYWl0IHVudGlsIHNldCB0aW1lIGJlZm9yZSBldmFsdWF0aW5nIG90aGVyIHRyaWdnZX'
    b'IgY29uZGl0aW9ucywgaW5jbHVkaW5nIGEgcmVjb3JkaW5nIGRlbGF5QBGuUAWUUmVjb3Jk'
    b'aW5nIFRpbWUgTGltaXRQAYMN/39RIYT////wUCWHU2Vjb25kc0ARQJBQBZhSZWNvcmRpbm'
    b'cgRGVsYXkvSW50ZXJ2YWxQAYMM/39RAYEAUCWHU2Vjb25kc1EhgwFRgFAV2FRpbWUgYmVm'
    b'b3JlIHN0YXJ0aW5nIGEgcmVjb3JkaW5nLCBhbmQgdGltZSBiZXR3ZWVuIHJlY29yZGluZ3'
    b'MgaWYgJ1JldHJpZ2dlcicgaXMgY2hlY2tlZC5AEa9QBZlSZWNvcmRpbmcgRmlsZSBTaXpl'
    b'IExpbWl0UAGDEf9/USGE////8FAlg0tpQkAQQJdQBYlSZXRyaWdnZXJQAYMO/39QFLBDb2'
    b'5maWdbMHhERkY3Rl09PW51bGwgYW5kIENvbmZpZ1sweDExRkY3Rl09PW51bGxQFc9XaGVu'
    b'IHNldCwgU2xhbSBTdGljayB3aWxsIHJlLWFybSBhZnRlciByZWNvcmRpbmcgdGltZSBvci'
    b'BzaXplIGxpbWl0IGlzIHJlYWNoZWQuQBBAqFAFnldhaXQgZm9yIEFsbCBTZW5zb3IgQ29u'
    b'ZGl0aW9uc1ABgxP/f1AV/ldoZW4gc2V0LCByZWNvcmRpbmcgd2lsbCBvbmx5IHN0YXJ0IH'
    b'doZW4gYWxsIHRyaWdnZXJzIGFyZSBtZXQsIG90aGVyd2lzZSByZWNvcmRpbmcgd2lsbCBz'
    b'dGFydCB3aGVuIGFueSBzZW5zb3IgY29uZGl0aW9uIGlzIG1ldEAYQO5QBZBQcmVzc3VyZS'
    b'BUcmlnZ2VyUAGDBQAkUQCBAECC5VAFlVByZXNzdXJlIFRyaWdnZXIsIExvd1ABgwMAJFEB'
    b'gwFfkFAlglBhUSGDAdTAUBWzU2V0IHRvIHN0YXJ0IHNhbXBsaW5nIHdoZW4gb3V0c2lkZS'
    b'BwcmVzc3VyZSB3aW5kb3cuQILmUAWWUHJlc3N1cmUgVHJpZ2dlciwgSGlnaFABgwQAJFEB'
    b'gwGtsFAlglBhUSGDAdTAUBWzU2V0IHRvIHN0YXJ0IHNhbXBsaW5nIHdoZW4gb3V0c2lkZS'
    b'BwcmVzc3VyZSB3aW5kb3cuQBhA+1AFk1RlbXBlcmF0dXJlIFRyaWdnZXJQAYMFASRRAIEA'
    b'QGPqUAWYVGVtcGVyYXR1cmUgVHJpZ2dlciwgTG93UAGDAwEkUQKB9lAlgUNREoHYUSKBUF'
    b'AVtlNldCB0byBzdGFydCBzYW1wbGluZyB3aGVuIG91dHNpZGUgdGVtcGVyYXR1cmUgd2lu'
    b'ZG93LkBj61AFmVRlbXBlcmF0dXJlIFRyaWdnZXIsIEhpZ2hQAYMEASRRAoEjUCWBQ1ESgd'
    b'hRIoFQUBW2U2V0IHRvIHN0YXJ0IHNhbXBsaW5nIHdoZW4gb3V0c2lkZSB0ZW1wZXJhdHVy'
    b'ZSB3aW5kb3cuQBhBqFAFm0FjY2VsZXJhdGlvbiBUcmlnZ2VyICg1MDBnKVABgwX/CFARgw'
    b'f/IFEAgQBQJIBQFJVDb25maWdbMHgwMUZGMDhdID09IDBAQ0CrUROIwH9AAAAAAABRI4hA'
    b'f0AAAAAAAFAFmUFjY2VsZXJhdGlvbiBUcmlnZ2VyLCBMb3dQAYMD/whRAoL5mlAV4EhpZ2'
    b'ggcG93ZXIgdHJpZ2dlciB3aXRoIHByZS10cmlnZ2VyIGRhdGEuIE5vdCByZWNvbW1lbmRl'
    b'ZCBmb3IgbG9uZyB3YWl0cyB3aXRob3V0IGV4dGVybmFsIHBvd2VyLlIDiD+PPHYmnaAlQE'
    b'NArFETiMB/QAAAAAAAUSOIQH9AAAAAAABQBZpBY2NlbGVyYXRpb24gVHJpZ2dlciwgSGln'
    b'aFABgwT/CFECggZmUBXgSGlnaCBwb3dlciB0cmlnZ2VyIHdpdGggcHJlLXRyaWdnZXIgZG'
    b'F0YS4gTm90IHJlY29tbWVuZGVkIGZvciBsb25nIHdhaXRzIHdpdGhvdXQgZXh0ZXJuYWwg'
    b'cG93ZXIuUgOIP488diadoCVAGEGcUAWaQWNjZWxlcmF0aW9uIFRyaWdnZXIgKDE2ZylQAY'
    b'MH/yBQFdRMb3cgcG93ZXIgdHJpZ2dlciB1c2luZyBkaWdpdGFsIGFjY2VsZXJvbWV0ZXIg'
    b'd2l0aCBEQyByZXNwb25zZS4gTm8gcHJlLXRyaWdnZXIgZGF0YS5RAIEAUBGDBf8IUCSAUB'
    b'SUQ29uZmlnWzB4MWZmMjBdID09IDBAQ0CxUAWWQWNjZWxlcmF0aW9uIFRocmVzaG9sZFAB'
    b'gwT/IFEBgVBSA4g/sBAQEBAQEFEjiEAwAAAAAAAAUROAUBXyTG93IHBvd2VyIHRyaWdnZX'
    b'IuIFRoZSBtaW5pbXVtIGFjY2VsZXJhdGlvbiAocG9zaXRpdmUgb3IgbmVnYXRpdmUpIHRv'
    b'IHRyaWdnZXIgYSByZWNvcmRpbmcuIE11c3QgYmUgZ3JlYXRlciB0aGFuIDAuQCfGUAGDBf'
    b'8gUQGBB0EHkVAFjlggQXhpcyBUcmlnZ2VyQQeRUAWOWSBBeGlzIFRyaWdnZXJBB5FQBY5a'
    b'IEF4aXMgVHJpZ2dlckAPgEBPgEAoRpVQBYxNZWFzdXJlbWVudHNACEJBUAWYNTAwZyBBY2'
    b'NlbGVyYXRpb24gKENoIDgpUCSAQCdAgVEBgQdQAYMB/whQFaBFbmFibGUvRGlzYWJsZSBh'
    b'Y2NlbGVyYXRpb24gYXhlc0EHmVAFlkVuYWJsZSBYIEF4aXMgKENoIDguMClBB5lQBZZFbm'
    b'FibGUgWSBBeGlzIChDaCA4LjEpQQeZUAWWRW5hYmxlIFogQXhpcyAoQ2ggOC4yKUARQJpQ'
    b'BYtTYW1wbGUgUmF0ZVABgwL/CFAV2FNldCBtYWluIGFjY2VsZXJhdGlvbiBzYW1wbGUgcm'
    b'F0ZSBmcm9tIDEwIHRvIDIwMDAwIEh6LgpIaWdoZXIgcmF0ZSByZWR1Y2VzIGJhdHRlcnkg'
    b'bGlmZS5QFJVDb25maWdbMHgwMUZGMDhdID09IDBRAYITiFAlgkh6URGBClEhgk4gQBFA/F'
    b'AFo092ZXJyaWRlIEFudGlhbGlhc2luZyBGaWx0ZXIgQ3V0b2ZmUAGDCP8IUQGCA+hQJYJI'
    b'elEhghOIUBVApURlZmF1bHQsIHdoZW4gbm90IGNoZWNrZWQsIG9mIG9uZS1maWZ0aCBzYW'
    b'1wbGUgZnJlcXVlbmN5LiBDdXQtb2ZmIGZyZXF1ZW5jeSBoYXMgMjklIGF0dGVudWF0aW9u'
    b'Lgo1dGggb3JkZXIgQnV0dGVyd29ydGguIFNvbWUgYXR0ZW51YXRpb24gYXQgNjAlIG9mIG'
    b'N1dC1vZmYgZnJlcXVlbmN5LlAUlUNvbmZpZ1sweDAxRkYwOF0gPT0gMEAIQLpQBZsxNmcg'
    b'REMgQWNjZWxlcmF0aW9uIChDaCAzMilAJ59QAYMB/yBRAYEBQQeSUAWPRW5hYmxlIEFsbC'
    b'BBeGVzQBf3UAWLU2FtcGxlIFJhdGVQAYMC/yBRAYIBkFAlgkh6UBSTbm90IENvbmZpZ1sw'
    b'eDFmZjIwXUEHhVEBggyAQQeFUQGCBkBBB4VRAYIDIEEHhVEBggGQQQeEUQGByEEHhFEBgW'
    b'RBB4RRAYEyQQeEUQGBGUEHhFEBgQxACEEwUAWoQ29udHJvbCBQYWQgVGVtcGVyYXR1cmUv'
    b'UHJlc3N1cmUgKENoIDU5KUAnQJtRAYEDUAGDAf87UBWlRW5hYmxlL0Rpc2FibGUgc2Vuc2'
    b'9ycyBvbiBjb250cm9sIHBhZEEHnFAFmUVuYWJsZSBQcmVzc3VyZSAoQ2ggNTkuMClBB59Q'
    b'BZxFbmFibGUgVGVtcGVyYXR1cmUgKENoIDU5LjEpQQelUAWiRW5hYmxlIFJlbGF0aXZlIE'
    b'h1bWlkaXR5IChDaCA1OS4yKUAR41AFi1NhbXBsZSBSYXRlUAGDAv87UBSUQ29uZmlnWzB4'
    b'MWZmM0JdID09IDBQFaRTZXQgc2FtcGxlIGZyZXF1ZW5jeSBmcm9tIDEgdG8gMTAgSHpRAY'
    b'EKUCWCSHpREYEBUSGBCkAIQclQBZlJbmVydGlhbCBNZWFzdXJlbWVudCBVbml0QAdAklAF'
    b'kEFjcXVpc2l0aW9uIE1vZGVQAYMB/ytRAYEQQQeKUAWDT2ZmUQGBAEEHqFAFoUFic29sdX'
    b'RlIE9yaWVudGF0aW9uIChRdWF0ZXJuaW9uKVEBgQhBB6hQBaFSZWxhdGl2ZSBPcmllbnRh'
    b'dGlvbiAoUXVhdGVybmlvbilRAYEQQQePUAWIUm90YXRpb25RAYECQAFAhlAFlU9yaWVudG'
    b'F0aW9uIERhdGEgUmF0ZVABgwIIK1EBgWRQJYJIelERgQFRIYHIUBW9U2FtcGxlIHJhdGUg'
    b'Zm9yIGRpcmVjdGlvbmFsIGRhdGEgZnJvbSAxIHRvIDIwMCBIei4gQ2hhbm5lbCA2NVAUlE'
    b'NvbmZpZ1sweDAxRkYyQl0gPCA4QAFAiVAFklJvdGF0aW9uIERhdGEgUmF0ZVABgwICK1EB'
    b'gWRQJYJIelERgQFRIYHIUBW+U2V0IHNhbXBsZSByYXRlIGZvciByb3RhdGlvbiBkYXRhIG'
    b'Zyb20gMSB0byAyMDAgSHouIENoYW5uZWwgNDdQFJlDb25maWdbMHgwMUZGMkJdICYgMiA9'
    b'PSAwQAj5UAWgQ29udHJvbCBQYWQgTGlnaHQgU2Vuc29yIChDaCA3NilAJ9NRAYEBUAGDAf'
    b'9MUBWjRW5hYmxlL0Rpc2FibGUgbGlnaHQgc2Vuc29yIGF0IDQgSHpBB6BQBZ1FbmFibGUg'
    b'TGlnaHQgU2Vuc29yIChDaCA3Ni4wKUAPgEBPgEoIgEoogEpIgA=='
)



"""
Default ConfigUI for unrecognized data recorders. It contains only the
UI items known to be universal (user-specified name, description,
and UTC offset widgets, and the calibration and info tabs.
"""

from base64 import b64decode

DEFAULT_CONFIG_UI = b64decode(
    b'GkXfoxAAAClChoEBQveBAULygQRC84EIQoKObWlkZS5zcy5jb25maWdCh4ECQoWBAnd3EA'
    b'ABk0AoEAABe1AFh0dlbmVyYWxABRAAAHNQBYtEZXZpY2UgTmFtZVABgwj/f1EkgUBQFdhB'
    b'IGN1c3RvbSBuYW1lIGZvciB0aGUgcmVjb3JkZXIuIE5vdCB0aGUgc2FtZSBhcyB0aGUgdm'
    b'9sdW1lIGxhYmVsLiA2NCBjaGFyYWN0ZXJzIG1heGltdW0uQAUQAABzUAWMRGV2aWNlIE5v'
    b'dGVzUAGDCf9/USSCAQBQFdJDdXN0b20gbm90ZXMgYWJvdXQgdGhlIHJlY29yZGVyIChwb3'
    b'NpdGlvbiwgdXNlciBJRCwgZXRjLikuIDI1NiBjaGFyYWN0ZXJzIG1heGltdW0uUTSBA0Aj'
    b'EAAAZ1ABgwv/f1AV3kVudGVyIGxvY2FsIHRpbWV6b25lJ3Mgb2Zmc2V0IGZyb20gVVRDIH'
    b'RpbWUuIEltcGFjdHMgdGltZXN0YW1wIGluZm9ybWF0aW9uIG9uIHJlY29yZGluZyBmaWxl'
    b'cy5ALxAAAABADxAAAABATxAAAABKCBAAAABKKBAAAABKSBAAAAA='
)



"""
Default ConfigUI for S5-E500D40 data recorders 
"""

from base64 import b64decode

DEFAULT_CONFIG_UI = b64decode(
    b'd3dVh0AoRZNQBYdHZW5lcmFsQAXzUAWLRGV2aWNlIE5hbWVQAYMI/39RJIFAUBXYQSBjdX'
    b'N0b20gbmFtZSBmb3IgdGhlIHJlY29yZGVyLiBOb3QgdGhlIHNhbWUgYXMgdGhlIHZvbHVt'
    b'ZSBsYWJlbC4gNjQgY2hhcmFjdGVycyBtYXhpbXVtLkAF81AFjERldmljZSBOb3Rlc1ABgw'
    b'n/f1EkggEAUBXSQ3VzdG9tIG5vdGVzIGFib3V0IHRoZSByZWNvcmRlciAocG9zaXRpb24s'
    b'IHVzZXIgSUQsIGV0Yy4pLiAyNTYgY2hhcmFjdGVycyBtYXhpbXVtLlE0gQNABftQBZNSZW'
    b'NvcmRpbmcgRGlyZWN0b3J5UAGDFP9/UQSGUkVDT1JEUSSBEFAUk0NvbmZpZ1sweDE2RkY3'
    b'Rl09PTBQFblEaXJlY3RvcnkgZm9yIHNlcmlhbGl6ZWQgcmVjb3JkaW5ncy4gMTYgY2hhcm'
    b'FjdGVyIG1heGltdW1ABUCjUAWVUmVjb3JkaW5nIEZpbGUgUHJlZml4UAGDFf9/USSBEFAV'
    b'6FByZWZpeCBmb3Igc2VyaWFsaXplZCByZWNvcmRpbmcgbmFtZXMuIElmIGJsYW5rLCBkZX'
    b'ZpY2Ugc2VyaWFsIG51bWJlciB3aWxsIGJlIHVzZWQuIDE2IGNoYXJhY3RlciBtYXhpbXVt'
    b'UBSTQ29uZmlnWzB4MTZGRjdGXT09MEAHQTxQBY9GaWxlIE5hbWUgU3R5bGVQAYMW/39RAY'
    b'EBUBVA/FRpbWUtQmFzZWQgZmlsZSBuYW1lcyB1c2UgYSBkaXJlY3RvcnkgYmFzZWQgb24g'
    b'dGhlIGRhdGUsIGFuZCBmaWxlIG5hbWUgYmFzZWQgb24gdGhlIHNlY29uZHMgc2luY2UgbW'
    b'lkbmlnaHQuClNlcmlhbGl6ZWQgZmlsZSBuYW1lcyB1c2UgdGhlIHNwZWNpZmllZCBkaXJl'
    b'Y3RvcnkgYW5kIGZpbGUgbmFtZSBwcmVmaXgsIGZvbGxvd2VkIGJ5IGEgbnVtYmVyLiAKTn'
    b'VtYmVyIGlzIHJlc2V0IHdoZW4gZGlyZWN0b3J5IG9yIHByZWZpeCBjaGFuZ2VzLkEHjVAF'
    b'ilRpbWUgQmFzZWRBB41QBYpTZXJpYWxpemVkQAdAqVAFi0J1dHRvbiBNb2RlUAGDEP9/UQ'
    b'GBAFAVv09uZSBTZWNvbmQgUHJlc3Mgd2lsbCBkaXNhYmxlIDMtc2Vjb25kIHByZXNzIGZv'
    b'ciBiYXR0ZXJ5IGNoZWNrLkEHl1AFlEltbWVkaWF0ZSBTdGFydC9TdG9wQQeiUAWfT25lIF'
    b'NlY29uZCBQcmVzcyBmb3IgU3RhcnQvU3RvcEEHjVAFilN0YXJ0LU9ubHlAF0EUUAWOUGx1'
    b'Zy1JbiBBY3Rpb25QAYMK/39RAYEBUBWtU2V0IGFjdGlvbiBmb3IgZGV2aWNlIHdoZW4gcG'
    b'x1Z2dlZCBpbiB0byBVU0IuQQe1UAWySW1tZWRpYXRlbHkgc3RvcCByZWNvcmRpbmcgYW5k'
    b'IGFwcGVhciBhcyBVU0IgZHJpdmVBB7NQBbBDb21wbGV0ZSByZWNvcmRpbmcgYmVmb3JlIG'
    b'FwcGVhcmluZyBhcyBVU0IgZHJpdmVBB7BQBa1JZ25vcmU6IHN0b3AgcmVjb3JkaW5nIHdo'
    b'ZW4gYnV0dG9uIGlzIHByZXNzZWRBB6VQBaJTdGFydCByZWNvcmRpbmcgd2hlbiBVU0IgY2'
    b'9ubmVjdGVkQCPnUAGDC/9/UBXeRW50ZXIgbG9jYWwgdGltZXpvbmUncyBvZmZzZXQgZnJv'
    b'bSBVVEMgdGltZS4gSW1wYWN0cyB0aW1lc3RhbXAgaW5mb3JtYXRpb24gb24gcmVjb3JkaW'
    b'5nIGZpbGVzLkAvgEAPgEBPgEAoSOhQBYhUcmlnZ2Vyc0AHQNpQBYxUcmlnZ2VyIE1vZGVQ'
    b'AYMS/39RAYEAUBVAj0RlbGF5IFRoZW4gVHJpZ2dlcjogV2FpdCBzcGVjaWZpZWQgdGltZS'
    b'wgdGhlbiB3YWl0IGZvciB0cmlnZ2VyCkRlbGF5IE9yIFRyaWdnZXI6IEFjcXVpcmUgYWZ0'
    b'ZXIgd2FpdCBvciBvbiB0cmlnZ2VyLiBCYXR0ZXJ5IGxpZmUgbWF5IGJlIHJlZHVjZWQuQQ'
    b'eVUAWSRGVsYXkgVGhlbiBUcmlnZ2VyQQeTUAWQRGVsYXkgT3IgVHJpZ2dlckAy9FAFjVN0'
    b'YXJ0IGF0IFRpbWVQAYMP/39QFdtXYWl0IHVudGlsIHNldCB0aW1lIGJlZm9yZSBldmFsdW'
    b'F0aW5nIG90aGVyIHRyaWdnZXIgY29uZGl0aW9ucywgaW5jbHVkaW5nIGEgcmVjb3JkaW5n'
    b'IGRlbGF5QBGuUAWUUmVjb3JkaW5nIFRpbWUgTGltaXRQAYMN/39RIYT////wUCWHU2Vjb2'
    b'5kc0ARQJBQBZhSZWNvcmRpbmcgRGVsYXkvSW50ZXJ2YWxQAYMM/39RAYEAUCWHU2Vjb25k'
    b'c1EhgwFRgFAV2FRpbWUgYmVmb3JlIHN0YXJ0aW5nIGEgcmVjb3JkaW5nLCBhbmQgdGltZS'
    b'BiZXR3ZWVuIHJlY29yZGluZ3MgaWYgJ1JldHJpZ2dlcicgaXMgY2hlY2tlZC5AEa9QBZlS'
    b'ZWNvcmRpbmcgRmlsZSBTaXplIExpbWl0UAGDEf9/USGE////8FAlg0tpQkAQQJdQBYlSZX'
    b'RyaWdnZXJQAYMO/39QFLBDb25maWdbMHhERkY3Rl09PW51bGwgYW5kIENvbmZpZ1sweDEx'
    b'RkY3Rl09PW51bGxQFc9XaGVuIHNldCwgU2xhbSBTdGljayB3aWxsIHJlLWFybSBhZnRlci'
    b'ByZWNvcmRpbmcgdGltZSBvciBzaXplIGxpbWl0IGlzIHJlYWNoZWQuQBBAqFAFnldhaXQg'
    b'Zm9yIEFsbCBTZW5zb3IgQ29uZGl0aW9uc1ABgxP/f1AV/ldoZW4gc2V0LCByZWNvcmRpbm'
    b'cgd2lsbCBvbmx5IHN0YXJ0IHdoZW4gYWxsIHRyaWdnZXJzIGFyZSBtZXQsIG90aGVyd2lz'
    b'ZSByZWNvcmRpbmcgd2lsbCBzdGFydCB3aGVuIGFueSBzZW5zb3IgY29uZGl0aW9uIGlzIG'
    b'1ldEAYQO5QBZBQcmVzc3VyZSBUcmlnZ2VyUAGDBQAkUQCBAECC5VAFlVByZXNzdXJlIFRy'
    b'aWdnZXIsIExvd1ABgwMAJFEBgwFfkFAlglBhUSGDAdTAUBWzU2V0IHRvIHN0YXJ0IHNhbX'
    b'BsaW5nIHdoZW4gb3V0c2lkZSBwcmVzc3VyZSB3aW5kb3cuQILmUAWWUHJlc3N1cmUgVHJp'
    b'Z2dlciwgSGlnaFABgwQAJFEBgwGtsFAlglBhUSGDAdTAUBWzU2V0IHRvIHN0YXJ0IHNhbX'
    b'BsaW5nIHdoZW4gb3V0c2lkZSBwcmVzc3VyZSB3aW5kb3cuQBhA+1AFk1RlbXBlcmF0dXJl'
    b'IFRyaWdnZXJQAYMFASRRAIEAQGPqUAWYVGVtcGVyYXR1cmUgVHJpZ2dlciwgTG93UAGDAw'
    b'EkUQKB9lAlgUNREoHYUSKBUFAVtlNldCB0byBzdGFydCBzYW1wbGluZyB3aGVuIG91dHNp'
    b'ZGUgdGVtcGVyYXR1cmUgd2luZG93LkBj61AFmVRlbXBlcmF0dXJlIFRyaWdnZXIsIEhpZ2'
    b'hQAYMEASRRAoEjUCWBQ1ESgdhRIoFQUBW2U2V0IHRvIHN0YXJ0IHNhbXBsaW5nIHdoZW4g'
    b'b3V0c2lkZSB0ZW1wZXJhdHVyZSB3aW5kb3cuQBhBqFAFm0FjY2VsZXJhdGlvbiBUcmlnZ2'
    b'VyICg1MDBnKVABgwX/CFARgwf/IFEAgQBQJIBQFJVDb25maWdbMHgwMUZGMDhdID09IDBA'
    b'Q0CrUROIwH9AAAAAAABRI4hAf0AAAAAAAFAFmUFjY2VsZXJhdGlvbiBUcmlnZ2VyLCBMb3'
    b'dQAYMD/whRAoL5mlAV4EhpZ2ggcG93ZXIgdHJpZ2dlciB3aXRoIHByZS10cmlnZ2VyIGRh'
    b'dGEuIE5vdCByZWNvbW1lbmRlZCBmb3IgbG9uZyB3YWl0cyB3aXRob3V0IGV4dGVybmFsIH'
    b'Bvd2VyLlIDiD+PQB9AH0AfQENArFETiMB/QAAAAAAAUSOIQH9AAAAAAABQBZpBY2NlbGVy'
    b'YXRpb24gVHJpZ2dlciwgSGlnaFABgwT/CFECggZmUBXgSGlnaCBwb3dlciB0cmlnZ2VyIH'
    b'dpdGggcHJlLXRyaWdnZXIgZGF0YS4gTm90IHJlY29tbWVuZGVkIGZvciBsb25nIHdhaXRz'
    b'IHdpdGhvdXQgZXh0ZXJuYWwgcG93ZXIuUgOIP49AH0AfQB9AGEGjUAWaQWNjZWxlcmF0aW'
    b'9uIFRyaWdnZXIgKDQwZylQAYMH/1BQFdRMb3cgcG93ZXIgdHJpZ2dlciB1c2luZyBkaWdp'
    b'dGFsIGFjY2VsZXJvbWV0ZXIgd2l0aCBEQyByZXNwb25zZS4gTm8gcHJlLXRyaWdnZXIgZG'
    b'F0YS5RAIEAUBGDBf8IUBGDB/8gUCSAUBSUQ29uZmlnWzB4MWZmNTBdID09IDBAQ0CyUAWW'
    b'QWNjZWxlcmF0aW9uIFRocmVzaG9sZFABgwT/UFEBgiAAUgOIP0QAFAAUABRRI4hARAAAAA'
    b'AAAFETgFAV8kxvdyBwb3dlciB0cmlnZ2VyLiBUaGUgbWluaW11bSBhY2NlbGVyYXRpb24g'
    b'KHBvc2l0aXZlIG9yIG5lZ2F0aXZlKSB0byB0cmlnZ2VyIGEgcmVjb3JkaW5nLiBNdXN0IG'
    b'JlIGdyZWF0ZXIgdGhhbiAwLkAnxlABgwX/UFEBgQdBB5FQBY5YIEF4aXMgVHJpZ2dlckEH'
    b'kVAFjlkgQXhpcyBUcmlnZ2VyQQeRUAWOWiBBeGlzIFRyaWdnZXJAD4BAT4BAKEb3UAWMTW'
    b'Vhc3VyZW1lbnRzQAhCQVAFmDUwMGcgQWNjZWxlcmF0aW9uIChDaCA4KVAkgEAnQIFRAYEH'
    b'UAGDAf8IUBWgRW5hYmxlL0Rpc2FibGUgYWNjZWxlcmF0aW9uIGF4ZXNBB5lQBZZFbmFibG'
    b'UgWCBBeGlzIChDaCA4LjApQQeZUAWWRW5hYmxlIFkgQXhpcyAoQ2ggOC4xKUEHmVAFlkVu'
    b'YWJsZSBaIEF4aXMgKENoIDguMilAEUCaUAWLU2FtcGxlIFJhdGVQAYMC/whQFdhTZXQgbW'
    b'FpbiBhY2NlbGVyYXRpb24gc2FtcGxlIHJhdGUgZnJvbSAxMCB0byAyMDAwMCBIei4KSGln'
    b'aGVyIHJhdGUgcmVkdWNlcyBiYXR0ZXJ5IGxpZmUuUBSVQ29uZmlnWzB4MDFGRjA4XSA9PS'
    b'AwUQGCE4hQJYJIelERgQpRIYJOIEARQPxQBaNPdmVycmlkZSBBbnRpYWxpYXNpbmcgRmls'
    b'dGVyIEN1dG9mZlABgwj/CFEBggPoUCWCSHpRIYJhqFAVQKVEZWZhdWx0LCB3aGVuIG5vdC'
    b'BjaGVja2VkLCBvZiBvbmUtZmlmdGggc2FtcGxlIGZyZXF1ZW5jeS4gQ3V0LW9mZiBmcmVx'
    b'dWVuY3kgaGFzIDI5JSBhdHRlbnVhdGlvbi4KNXRoIG9yZGVyIEJ1dHRlcndvcnRoLiBTb2'
    b'1lIGF0dGVudWF0aW9uIGF0IDYwJSBvZiBjdXQtb2ZmIGZyZXF1ZW5jeS5QFJVDb25maWdb'
    b'MHgwMUZGMDhdID09IDBACEEcUAWbNDBnIERDIEFjY2VsZXJhdGlvbiAoQ2ggODApQCefUA'
    b'GDAf9QUQGBAUEHklAFj0VuYWJsZSBBbGwgQXhlc0AXQNhQBYtTYW1wbGUgUmF0ZVABgwL/'
    b'UFEBggH0UCWCSHpQFd5Mb3cgcGFzcyBmaWx0ZXIgaXMgc2V0IHRvIDI1JSBvZiBkYXRhIH'
    b'JhdGUsIHNvIGEgZGF0YSByYXRlIG9mIDIwMDBIeiBoYXMgYSBMUCBmaWx0ZXIgYXQgNTAw'
    b'IEh6UBSTbm90IENvbmZpZ1sweDFmZjUwXUEHhVEBgg+gQQeFUQGCB9BBB4VRAYID6EEHhV'
    b'EBggH0QQeEUQGB+kEHhFEBgX1BB4RRAYE/QQeEUQGBIEEHhFEBgRBACEEwUAWoQ29udHJv'
    b'bCBQYWQgVGVtcGVyYXR1cmUvUHJlc3N1cmUgKENoIDU5KUAnQJtRAYEDUAGDAf87UBWlRW'
    b'5hYmxlL0Rpc2FibGUgc2Vuc29ycyBvbiBjb250cm9sIHBhZEEHnFAFmUVuYWJsZSBQcmVz'
    b'c3VyZSAoQ2ggNTkuMClBB59QBZxFbmFibGUgVGVtcGVyYXR1cmUgKENoIDU5LjEpQQelUA'
    b'WiRW5hYmxlIFJlbGF0aXZlIEh1bWlkaXR5IChDaCA1OS4yKUAR41AFi1NhbXBsZSBSYXRl'
    b'UAGDAv87UBSUQ29uZmlnWzB4MWZmM0JdID09IDBQFaRTZXQgc2FtcGxlIGZyZXF1ZW5jeS'
    b'Bmcm9tIDEgdG8gMTAgSHpRAYEKUCWCSHpREYEBUSGBCkAIQc1QBZlJbmVydGlhbCBNZWFz'
    b'dXJlbWVudCBVbml0QAdAllAFkEFjcXVpc2l0aW9uIE1vZGVQAYMB/ytQJYEgUQGBEEEHil'
    b'AFg09mZlEBgQBBB6hQBaFBYnNvbHV0ZSBPcmllbnRhdGlvbiAoUXVhdGVybmlvbilRAYEI'
    b'QQeoUAWhUmVsYXRpdmUgT3JpZW50YXRpb24gKFF1YXRlcm5pb24pUQGBEEEHj1AFiFJvdG'
    b'F0aW9uUQGBAkABQIZQBZVPcmllbnRhdGlvbiBEYXRhIFJhdGVQAYMCCCtRAYFkUCWCSHpR'
    b'EYEBUSGByFAVvVNhbXBsZSByYXRlIGZvciBkaXJlY3Rpb25hbCBkYXRhIGZyb20gMSB0by'
    b'AyMDAgSHouIENoYW5uZWwgNjVQFJRDb25maWdbMHgwMUZGMkJdIDwgOEABQIlQBZJSb3Rh'
    b'dGlvbiBEYXRhIFJhdGVQAYMCAitRAYFkUCWCSHpREYEBUSGByFAVvlNldCBzYW1wbGUgcm'
    b'F0ZSBmb3Igcm90YXRpb24gZGF0YSBmcm9tIDEgdG8gMjAwIEh6LiBDaGFubmVsIDQ3UBSZ'
    b'Q29uZmlnWzB4MDFGRjJCXSAmIDIgPT0gMEAI9VAFoENvbnRyb2wgUGFkIExpZ2h0IFNlbn'
    b'NvciAoQ2ggNzYpQCfPUAGDAf9MUBWjRW5hYmxlL0Rpc2FibGUgbGlnaHQgc2Vuc29yIGF0'
    b'IDQgSHpBB6BQBZ1FbmFibGUgTGlnaHQgU2Vuc29yIChDaCA3Ni4wKUAPgEBPgEoIgEoogE'
    b'pIgA=='
)



"""
Default ConfigUI for Sx-D16 data recorders 
"""

from base64 import b64decode

DEFAULT_CONFIG_UI = b64decode(
    b'GkXfo6lChoEBQveBAULygQRC84EIQoKObWlkZS5zcy5jb25maWdCh4ECQoWBAnd3EAATk0'
    b'AoIAYKUAWHR2VuZXJhbEAFQHNQBYtEZXZpY2UgTmFtZVABgwj/f1EkgUBQFdhBIGN1c3Rv'
    b'bSBuYW1lIGZvciB0aGUgcmVjb3JkZXIuIE5vdCB0aGUgc2FtZSBhcyB0aGUgdm9sdW1lIG'
    b'xhYmVsLiA2NCBjaGFyYWN0ZXJzIG1heGltdW0uQAVAc1AFjERldmljZSBOb3Rlc1ABgwn/'
    b'f1EkggEAUBXSQ3VzdG9tIG5vdGVzIGFib3V0IHRoZSByZWNvcmRlciAocG9zaXRpb24sIH'
    b'VzZXIgSUQsIGV0Yy4pLiAyNTYgY2hhcmFjdGVycyBtYXhpbXVtLlE0gQNAB0E8UAWPRmls'
    b'ZSBOYW1lIFN0eWxlUAGDFv9/UQGBAFAVQPxUaW1lLUJhc2VkIGZpbGUgbmFtZXMgdXNlIG'
    b'EgZGlyZWN0b3J5IGJhc2VkIG9uIHRoZSBkYXRlLCBhbmQgZmlsZSBuYW1lIGJhc2VkIG9u'
    b'IHRoZSBzZWNvbmRzIHNpbmNlIG1pZG5pZ2h0LgpTZXJpYWxpemVkIGZpbGUgbmFtZXMgdX'
    b'NlIHRoZSBzcGVjaWZpZWQgZGlyZWN0b3J5IGFuZCBmaWxlIG5hbWUgcHJlZml4LCBmb2xs'
    b'b3dlZCBieSBhIG51bWJlci4gCk51bWJlciBpcyByZXNldCB3aGVuIGRpcmVjdG9yeSBvci'
    b'BwcmVmaXggY2hhbmdlcy5BB41QBYpUaW1lIEJhc2VkQQeNUAWKU2VyaWFsaXplZEAFQHtQ'
    b'BZNSZWNvcmRpbmcgRGlyZWN0b3J5UAGDFP9/UQSGUkVDT1JEUSSBEFAUk0NvbmZpZ1sweD'
    b'E2RkY3Rl09PTBQFblEaXJlY3RvcnkgZm9yIHNlcmlhbGl6ZWQgcmVjb3JkaW5ncy4gMTYg'
    b'Y2hhcmFjdGVyIG1heGltdW1ABUCjUAWVUmVjb3JkaW5nIEZpbGUgUHJlZml4UAGDFf9/US'
    b'SBEFAV6FByZWZpeCBmb3Igc2VyaWFsaXplZCByZWNvcmRpbmcgbmFtZXMuIElmIGJsYW5r'
    b'LCBkZXZpY2Ugc2VyaWFsIG51bWJlciB3aWxsIGJlIHVzZWQuIDE2IGNoYXJhY3RlciBtYX'
    b'hpbXVtUBSTQ29uZmlnWzB4MTZGRjdGXT09MEAXQRRQBY5QbHVnLUluIEFjdGlvblABgwr/'
    b'f1EBgQFQFa1TZXQgYWN0aW9uIGZvciBkZXZpY2Ugd2hlbiBwbHVnZ2VkIGluIHRvIFVTQi'
    b'5BB7VQBbJJbW1lZGlhdGVseSBzdG9wIHJlY29yZGluZyBhbmQgYXBwZWFyIGFzIFVTQiBk'
    b'cml2ZUEHs1AFsENvbXBsZXRlIHJlY29yZGluZyBiZWZvcmUgYXBwZWFyaW5nIGFzIFVTQi'
    b'Bkcml2ZUEHsFAFrUlnbm9yZTogc3RvcCByZWNvcmRpbmcgd2hlbiBidXR0b24gaXMgcHJl'
    b'c3NlZEEHpVAFolN0YXJ0IHJlY29yZGluZyB3aGVuIFVTQiBjb25uZWN0ZWRAB0EcUAWLQn'
    b'V0dG9uIE1vZGVQAYMQ/39RAYEAUBVAwU9uZSBTZWNvbmQgUHJlc3Mgd2lsbCBkaXNhYmxl'
    b'IDMtc2Vjb25kIHByZXNzIGZvciBiYXR0ZXJ5IGNoZWNrLgpOT1RFOiBJZiAnU3RhcnQtT2'
    b'5seScgaXMgc2VsZWN0ZWQgYW5kIG5vIHJlY29yZGluZyB0aW1lIGxpbWl0IG9yIG1heCBm'
    b'aWxlIHNpemUgaXMgc2V0LCByZWNvcmRpbmdzIGNhbiBiZSBzdG9wcGVkIHdpdGggVVNCIH'
    b'BsdWctaW5BB5dQBZRJbW1lZGlhdGUgU3RhcnQvU3RvcEEHolAFn09uZSBTZWNvbmQgUHJl'
    b'c3MgZm9yIFN0YXJ0L1N0b3BAI0BnUAGDC/9/UBXeRW50ZXIgbG9jYWwgdGltZXpvbmUncy'
    b'BvZmZzZXQgZnJvbSBVVEMgdGltZS4gSW1wYWN0cyB0aW1lc3RhbXAgaW5mb3JtYXRpb24g'
    b'b24gcmVjb3JkaW5nIGZpbGVzLkAvgEAPgEBPgEAoIAgeUAWIVHJpZ2dlcnNAB0DaUAWMVH'
    b'JpZ2dlciBNb2RlUAGDEv9/UQGBAFAVQI9EZWxheSBUaGVuIFRyaWdnZXI6IFdhaXQgc3Bl'
    b'Y2lmaWVkIHRpbWUsIHRoZW4gd2FpdCBmb3IgdHJpZ2dlcgpEZWxheSBPciBUcmlnZ2VyOi'
    b'BBY3F1aXJlIGFmdGVyIHdhaXQgb3Igb24gdHJpZ2dlci4gQmF0dGVyeSBsaWZlIG1heSBi'
    b'ZSByZWR1Y2VkLkEHlVAFkkRlbGF5IFRoZW4gVHJpZ2dlckEHk1AFkERlbGF5IE9yIFRyaW'
    b'dnZXJAMkB0UAWNU3RhcnQgYXQgVGltZVABgw//f1AV21dhaXQgdW50aWwgc2V0IHRpbWUg'
    b'YmVmb3JlIGV2YWx1YXRpbmcgb3RoZXIgdHJpZ2dlciBjb25kaXRpb25zLCBpbmNsdWRpbm'
    b'cgYSByZWNvcmRpbmcgZGVsYXlAEUAuUAWUUmVjb3JkaW5nIFRpbWUgTGltaXRQAYMN/39R'
    b'IYT////wUCWHU2Vjb25kc0ARQJJQBZhSZWNvcmRpbmcgRGVsYXkvSW50ZXJ2YWxQAYMM/3'
    b'9RAYEAUCWHU2Vjb25kc1EhgwFRgFAV2kRlbGF5IGJlZm9yZSBzdGFydGluZyBhIHJlY29y'
    b'ZGluZywgaW5jbHVkaW5nIGJldHdlZW4gcmVjb3JkaW5ncyBpZiAnUmV0cmlnZ2VyJyBpcy'
    b'BjaGVja2VkLkARQC9QBZlSZWNvcmRpbmcgRmlsZSBTaXplIExpbWl0UAGDEf9/USGE////'
    b'8FAlg0tpQkAQQJJQBYlSZXRyaWdnZXJQAYMO/39QFLBDb25maWdbMHhERkY3Rl09PW51bG'
    b'wgYW5kIENvbmZpZ1sweDExRkY3Rl09PW51bGxQFcpXaGVuIHNldCwgZW5EQVEgd2lsbCBy'
    b'ZS1hcm0gYWZ0ZXIgcmVjb3JkaW5nIHRpbWUgb3Igc2l6ZSBsaW1pdCBpcyByZWFjaGVkLk'
    b'AQQKhQBZ5XYWl0IGZvciBBbGwgU2Vuc29yIENvbmRpdGlvbnNQAYMT/39QFf5XaGVuIHNl'
    b'dCwgcmVjb3JkaW5nIHdpbGwgb25seSBzdGFydCB3aGVuIGFsbCB0cmlnZ2VycyBhcmUgbW'
    b'V0LCBvdGhlcndpc2UgcmVjb3JkaW5nIHdpbGwgc3RhcnQgd2hlbiBhbnkgc2Vuc29yIGNv'
    b'bmRpdGlvbiBpcyBtZXRAGEFbUAWQUHJlc3N1cmUgVHJpZ2dlclABgwUAJFEAgQBQFOgoQ2'
    b'9uZmlnWzB4MWZmMTRdIGFuZCAoQ29uZmlnWzB4MWZmMTRdICAmIDEgPT0gMCkpIG9yIChD'
    b'b25maWdbMHgwMUZGMjRdIGFuZCAoQ29uZmlnWzB4MDFGRjI0XSAgJiAxID09IDApKUCCQG'
    b'VQBZVQcmVzc3VyZSBUcmlnZ2VyLCBMb3dQAYMDACRRAYMBX5BQJYJQYVEhgwHUwFAVs1Nl'
    b'dCB0byBzdGFydCBzYW1wbGluZyB3aGVuIG91dHNpZGUgcHJlc3N1cmUgd2luZG93LkCCQG'
    b'ZQBZZQcmVzc3VyZSBUcmlnZ2VyLCBIaWdoUAGDBAAkUQGDAa2wUCWCUGFRIYMB1MBQFbNT'
    b'ZXQgdG8gc3RhcnQgc2FtcGxpbmcgd2hlbiBvdXRzaWRlIHByZXNzdXJlIHdpbmRvdy5AGE'
    b'FoUAWTVGVtcGVyYXR1cmUgVHJpZ2dlclABgwUBJFEAgQBQFOgoQ29uZmlnWzB4MWZmMTRd'
    b'IGFuZCAoQ29uZmlnWzB4MWZmMTRdICAmIDIgPT0gMCkpIG9yIChDb25maWdbMHgwMUZGMj'
    b'RdIGFuZCAoQ29uZmlnWzB4MDFGRjI0XSAgJiAyID09IDApKUBjQGpQBZhUZW1wZXJhdHVy'
    b'ZSBUcmlnZ2VyLCBMb3dQAYMDASRRAoH2UCWBQ1ESgdhRIoFQUBW2U2V0IHRvIHN0YXJ0IH'
    b'NhbXBsaW5nIHdoZW4gb3V0c2lkZSB0ZW1wZXJhdHVyZSB3aW5kb3cuQGNAa1AFmVRlbXBl'
    b'cmF0dXJlIFRyaWdnZXIsIEhpZ2hQAYMEASRRAoEjUCWBQ1ESgdhRIoFQUBW2U2V0IHRvIH'
    b'N0YXJ0IHNhbXBsaW5nIHdoZW4gb3V0c2lkZSB0ZW1wZXJhdHVyZSB3aW5kb3cuQBhBq1AF'
    b'mkFjY2VsZXJhdGlvbiBUcmlnZ2VyICgxNmcpUAGDB/8gUBXUTG93IHBvd2VyIHRyaWdnZX'
    b'IgdXNpbmcgZGlnaXRhbCBhY2NlbGVyb21ldGVyIHdpdGggREMgcmVzcG9uc2UuIE5vIHBy'
    b'ZS10cmlnZ2VyIGRhdGEuUQCBAFARgwX/CFARgwf/UFAkgFAUlENvbmZpZ1sweDFmZjIwXS'
    b'A9PSAwQENAuVAFlkFjY2VsZXJhdGlvbiBUaHJlc2hvbGRQAYME/yBRAYFQUgOIP7AQEBAQ'
    b'EBBRI4hAMAAAAAAAAFETiAAAAAAAAAAAUBXyTG93IHBvd2VyIHRyaWdnZXIuIFRoZSBtaW'
    b'5pbXVtIGFjY2VsZXJhdGlvbiAocG9zaXRpdmUgb3IgbmVnYXRpdmUpIHRvIHRyaWdnZXIg'
    b'YSByZWNvcmRpbmcuIE11c3QgYmUgZ3JlYXRlciB0aGFuIDAuQCdARlABgwX/IFEBgQdBB5'
    b'FQBY5YIEF4aXMgVHJpZ2dlckEHkVAFjlkgQXhpcyBUcmlnZ2VyQQeRUAWOWiBBeGlzIFRy'
    b'aWdnZXJAD4BAT4BAKCAFU1AFjE1lYXN1cmVtZW50c0AIQLxQBZsxNmcgREMgQWNjZWxlcm'
    b'F0aW9uIChDaCAzMilAJ0AfUAGDAf8gUQGBAUEHklAFj0VuYWJsZSBBbGwgQXhlc0AXQHdQ'
    b'BYtTYW1wbGUgUmF0ZVABgwL/IFEBggGQUCWCSHpQFJNub3QgQ29uZmlnWzB4MWZmMjBdQQ'
    b'eFUQGCDIBBB4VRAYIGQEEHhVEBggMgQQeFUQGCAZBBB4RRAYHIQQeEUQGBZEEHhFEBgTJB'
    b'B4RRAYEZQQeEUQGBDEAIQTZQBahDb250cm9sIFBhZCBUZW1wZXJhdHVyZS9QcmVzc3VyZS'
    b'AoQ2ggNTkpQCdAoFEBgQNQAYMB/ztQFapFbmFibGUvRGlzYWJsZSBDb250cm9sIFBhZCBT'
    b'ZW5zb3IgQ2hhbm5lbHNBB5xQBZlFbmFibGUgUHJlc3N1cmUgKENoIDU5LjApQQefUAWcRW'
    b'5hYmxlIFRlbXBlcmF0dXJlIChDaCA1OS4xKUEHpVAFokVuYWJsZSBSZWxhdGl2ZSBIdW1p'
    b'ZGl0eSAoQ2ggNTkuMilAEUBjUAWLU2FtcGxlIFJhdGVQAYMC/ztQFJRDb25maWdbMHgxZm'
    b'YzQl0gPT0gMFAVpFNldCBzYW1wbGUgZnJlcXVlbmN5IGZyb20gMSB0byAxMCBIelEBgQpQ'
    b'JYJIelERgQFRIYEKQAhBzVAFmUluZXJ0aWFsIE1lYXN1cmVtZW50IFVuaXRAB0CWUAWQQW'
    b'NxdWlzaXRpb24gTW9kZVABgwH/K1AlgSBRAYEQQQeKUAWDT2ZmUQGBAEEHqFAFoUFic29s'
    b'dXRlIE9yaWVudGF0aW9uIChRdWF0ZXJuaW9uKVEBgQhBB6hQBaFSZWxhdGl2ZSBPcmllbn'
    b'RhdGlvbiAoUXVhdGVybmlvbilRAYEQQQePUAWIUm90YXRpb25RAYECQAFAhlAFlU9yaWVu'
    b'dGF0aW9uIERhdGEgUmF0ZVABgwIIK1EBgWRQJYJIelERgQFRIYHIUBW9U2FtcGxlIHJhdG'
    b'UgZm9yIGRpcmVjdGlvbmFsIGRhdGEgZnJvbSAxIHRvIDIwMCBIei4gQ2hhbm5lbCA2NVAU'
    b'lENvbmZpZ1sweDAxRkYyQl0gPCA4QAFAiVAFklJvdGF0aW9uIERhdGEgUmF0ZVABgwICK1'
    b'EBgWRQJYJIelERgQFRIYHIUBW+U2V0IHNhbXBsZSByYXRlIGZvciByb3RhdGlvbiBkYXRh'
    b'IGZyb20gMSB0byAyMDAgSHouIENoYW5uZWwgNDdQFJlDb25maWdbMHgwMUZGMkJdICYgMi'
    b'A9PSAwQAhA8VAFpUludGVybmFsIFRlbXBlcmF0dXJlL1ByZXNzdXJlIChDaCAzNilAJ0DF'
    b'UQGBA1ABgwH/JFAV90VuYWJsZS9EaXNhYmxlIHNsb3dlciBpbnRlcm5hbCBlbnZpcm9ubW'
    b'VudGFsIHNlbnNvcnMuIE5PVEU6IFRlbXBlcmF0dXJlIGlzIHJlcXVpcmVkIGZvciBtYWlu'
    b'IGFjY2VsZXJvbWV0ZXIgbWVhc3VyZW1lbnRzQQecUAWZRW5hYmxlIFByZXNzdXJlIChDaC'
    b'AzNi4wKUEHn1AFnEVuYWJsZSBUZW1wZXJhdHVyZSAoQ2ggMzYuMSlACEB6UAWgQ29udHJv'
    b'bCBQYWQgTGlnaHQgU2Vuc29yIChDaCA3NilAJ0BTUAGDAf9MUQGBAVAVo0VuYWJsZS9EaX'
    b'NhYmxlIGxpZ2h0IHNlbnNvciBhdCA0IEh6QQegUAWdRW5hYmxlIExpZ2h0IFNlbnNvciAo'
    b'Q2ggNzYuMClAD4BAT4BKCIBKKIBKSIA='
)
